/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstPackOprParam;
import com.abssqr.plat.common.model.domain.ast.AstPack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.ast.AstPackRepo;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.general.system.common.util.VarChecker;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.core.service.ast: AstPackManager.java, v 0.1 2020-04-18 4:16 PM xiachenxiang Exp $
 */
@Component
public class AstPackManager {
    @Autowired
    private AstPackRepo          astPackRepository;
    @Autowired
    private AstScreenRuleRepo    astScreenRuleRepo;
    @Autowired
    private CommonTaskRepository commonTaskRepository;
    @Autowired
    private AbsConfig            absConfig;

    /**
     * 新增资产包
     *
     * @param param
     * @return
     */
    public String add(AstPackOprParam param) {
        // 1. 校验重复
        AstPack astPack = astPackRepository.getByAbsPackPackNo(param.getPackNo());
        VarChecker.checkArgument(astPack == null, "资产包已存在");
        Long cnt = astPackRepository.cntByPackName(param.getPackName());
        VarChecker.checkEquals(cnt, 0L, "资产包名重复");
        // 2. 保存资产包
        astPack = buildPack(param);
        astPackRepository.insert(astPack);

        // 3. 保存筛选规则
        astScreenRuleRepo.insertBatch(astPack.getScreenRuleList());

        // 4. 插入资产筛选任务
        CommonTask commonTask = buildBasicCommonTask(param.getPackNo(), param.getAstPoolNo());
        commonTaskRepository.create(commonTask);

        return param.getPackNo();
    }


    private AstPack buildPack(AstPackOprParam param) {
        AstPack result = new AstPack();

        result.setPackNo(param.getPackNo());
        result.setPackName(param.getPackName());
        result.setAvalAmt(param.getAvalAmtMoney());
        result.setAstPoolNo(param.getAstPoolNo());
        result.setPackType(AstPackTypeEnum.NORMAL);
        result.setStatus(AstPackStatusEnum.INIT);
        result.setAstType(AstTypeEnum.getByCode(param.getAstType()));
        if (CollectionUtils.isNotEmpty(param.getScreenRuleList())) {
            param.getScreenRuleList().forEach(item -> {
                if (StringUtils.isNotEmpty(item.getLeftValue()) || StringUtils.isNotEmpty(item.getRightValue())) {
                    AstScreenRule rule = new AstScreenRule();
                    result.getScreenRuleList().add(rule);
                    //relNo 关联编号
                    rule.setRelNo(param.getPackNo());
                    //relType 关联类型 1.资产打包 PACK
                    rule.setRelType(AstScreenRelTypeEnum.PACK);
                    //ruleType 规则类型
                    rule.setRuleType(AstScreenRuleTypeEnum.getByCode(item.getRuleType()));
                    //leftValue 左值
                    rule.setLeftValue(item.getLeftValue());
                    //rightValue 右值
                    rule.setRightValue(item.getRightValue());
                    //leftCondition 左条件: 1.大于 GT 2.大于等于 GE
                    rule.setLeftCondition(AstScreenRuleConditionEnum.getByCode(item.getLeftCondition()));
                    //rightCondition 右条件: 1.小于 LT 2.小于等于 LE 3.等于 EQ 4.包含 IN
                    rule.setRightCondition(AstScreenRuleConditionEnum.getByCode(item.getRightCondition()));
                }
            });
        }
        return result;
    }

    private CommonTask buildBasicCommonTask(String packNo, String astPoolNo) {
        CommonTask commonTask = new CommonTask();
        commonTask.setBizNo(packNo);
        commonTask.setContext(astPoolNo);
        commonTask.setTaskType(CommonTaskTypeEnum.PACK_MATCH);
        commonTask.setTaskStatus(TaskStatusEnum.INIT);
        commonTask.setEnv(absConfig.getEnv());
        commonTask.setExecTimes(0L);
        return commonTask;
    }
}
